/**
 * 
 */
package de.mpiwg.itgroup.annotations.restlet.annotations_ui;

import java.util.List;

import org.apache.log4j.Logger;
import org.restlet.data.Form;
import org.restlet.data.MediaType;
import org.restlet.data.Reference;
import org.restlet.data.Status;
import org.restlet.representation.Representation;
import org.restlet.representation.StringRepresentation;
import org.restlet.resource.Get;
import org.restlet.resource.Post;
import org.restlet.resource.ServerResource;

import de.mpiwg.itgroup.annotations.Actor;
import de.mpiwg.itgroup.annotations.Group;
import de.mpiwg.itgroup.annotations.neo4j.AnnotationStore;
import de.mpiwg.itgroup.annotations.restlet.BaseRestlet;

/**
 * Resource class for the list of annotation groups.
 * 
 * @author casties
 * 
 */
public class GroupsResource extends ServerResource {

    public static Logger logger = Logger.getLogger(GroupsResource.class);

    private AnnotationStore store;

    /**
     * GET with HTML content type. Lists all groups.
     * 
     * @param entity
     * @return
     */
    @Get("html")
    public Representation doGetHTML(Representation entity) {
        String result = null;
        store = getAnnotationStore();
        // list all groups
        result = "<html><body>\n<h1>Groups</h1>\n<table>";
        result += "<tr><th>id</th><th>name</th><th>uri</th></tr>";
        List<Group> groups = store.getGroups("uri", "*");
        for (Group group : groups) {
            Reference groupUrl = this.getReference();
            groupUrl.addSegment(group.getId());
            result += String.format("<tr><td><a href=\"%s\">%s</a></td><td>%s</td><td>%s</td></tr>\n", groupUrl, group.getId(),
                    group.getName(), group.getUri());
        }
        result += "</table>\n</body>\n</html>";
        logger.debug("sending:");
        logger.debug(result);
        return new StringRepresentation(result, MediaType.TEXT_HTML);
    }

    /**
     * POST creates a new Group.
     * 
     * @return
     */
    @Post
    public Representation doPost(Representation entity) {
        logger.debug("AnnotationsUiGroups doPost!");
        // TODO: do authentication
        Form form = new Form(entity);
        String id = form.getFirstValue("id");
        String name = form.getFirstValue("name");
        if (id == null || id.isEmpty()) {
            // invalid id
            setStatus(Status.CLIENT_ERROR_BAD_REQUEST);
            return null;
        }
        String gid = makeGroupId(id);
        Group newGroup = new Group(gid, null, name);
        store = getAnnotationStore();
        Actor storedGroup = store.storeActor(newGroup);
        gid = storedGroup.getId();
        // return 303: see other
        setStatus(Status.REDIRECTION_SEE_OTHER);
        // go GET URL for this group
        Reference groupUrl = this.getReference();
        groupUrl.addSegment(gid);
        this.getResponse().setLocationRef(groupUrl);
        return null;
    }

    /**
     * Returns a group id based on the given id.
     * 
     * @param id
     * @return
     */
    protected String makeGroupId(String id) {
        // TODO: should we use different ids?
        id = id.replaceAll("\\W", "_");
        id = id.toLowerCase();
        return id;
    }

    protected AnnotationStore getAnnotationStore() {
        if (store == null) {
            store = ((BaseRestlet) getApplication()).getAnnotationStore();
        }
        return store;
    }
}
